////////////////////////////////////////////////////////////////////////////////
//	BaseSplitter.cpp:	The base (and only) class for splitting Blender export files
//
//	This class reads an Blender export text file containing the information
//	dumped from the objects in a Blender file, and a separate input control
//	text file that indicates the names of the individual Blender export file
//	to be created, and which objects go into which export file.  Some standard
//	information goes at the head of each export file, but the rest is made up of
//	the objects linked to that file.
//
//	This addresses the fact that the current Blender export facility does not
//	recognize "selected" objects, and is easier than having to select just the
//	right objects each time you want to export something anyway.
//
//	Copyright , Jack Emmerichs, 2001.
//
////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "FileSplitter.h"
#include "BaseSplitter.h"

#define	EQUAL 0


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

/*
 *	JFE 9/23/01.  Changed the input scanning to recognize the type of object now
 *	included in the comments box around the object name, and to ignore the new
 *	camera and lights object types that may be in the Blender export file.
 */

////////////////////////////////////////////////////////////////////////////////
// Construction/Destruction
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//	Default constructor
//
CBaseSplitter::CBaseSplitter()
{

}

////////////////////////////////////////////////////////////////////////////////
//	Standard destructor
//
CBaseSplitter::~CBaseSplitter()
{

}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Provides the main processing function of splitting the input file.
//
//   COMMENTS:
//
//	This method will perform the base functions of opening and closing the
//	requited files files (etc.).  Separate methods will read the input from the
//	appropriate files and write the output to the appropriate output file.
//
//	This function will not return until the entire input file has been processed.
//
//	  szFileName		The name of the input file to split
//	
//    RETURN VALUES:
//        None.  This method must handle all errors.
//
void CBaseSplitter::SplitInputFile(char *szFileName)
{
	strcpy (m_szInputFileName, szFileName);
	if (!OpenTheFiles())
	{
			CloseDown();
			return;
	}
//
//			Process the contents of the main Blender export file
//
	int	nResult	= 0;
	while (TRUE)
	{
		if (! ReadNextLine (m_pIFStream))							// can't read line
		{
			CloseDown();
			return;
		}
		else if (strnicmp (m_InputBuffer, "File Ends.", 10) == EQUAL)// Found EOF
		{
			if (!WriteToAllOutputFiles (m_InputBuffer))
				return;
			CloseDown();
		    MessageBox( NULL, "Split processing completed.", NULL, MB_OK );
			return;
		}
//
// JFE 9/23/01: Start changes to read the object type in the first line of
// the comment box, and to ignore camera and lamp type objects.
//
		else if (strnicmp (m_InputBuffer, "#mesh#", 6) == EQUAL)		// Found Start of a mesh
		{
			strcpy (m_szHoldMeshStart, m_InputBuffer);
			if (! ReadNextLine (m_pIFStream))
			{
				CloseDown();
			    MessageBox( NULL, "Can't read line after ###..., processing stops.", NULL, MB_OK );
				return;
			}
			nResult = sscanf (m_InputBuffer, "%*s %s %*s", m_szMeshObjectName);
			if (nResult != 1)
			{
			    MessageBox( NULL, "Can't find mesh name after ###..., processing stops.", NULL, MB_OK );
				CloseDown();
				return;
			}
			WriteOutMeshData (m_szMeshObjectName);
		}
		else if (strnicmp (m_InputBuffer, "#camera#", 8) == EQUAL)		// Found Start of a camera
		{
			while (strnicmp (m_InputBuffer, "Camera ends.", 12) != EQUAL)
			{
				if (! ReadNextLine (m_pIFStream))
				{
					CloseDown();
					MessageBox( NULL, "Can't read line in camera description, processing stops.", NULL, MB_OK );
					return;
				}
			}
		}
		else if (strnicmp (m_InputBuffer, "#lamp#", 6) == EQUAL)		// Found Start of a lamp
		{
			while (strnicmp (m_InputBuffer, "Lamp ends.", 10) != EQUAL)
			{
				if (! ReadNextLine (m_pIFStream))
				{
					CloseDown();
					MessageBox( NULL, "Can't read line in camera description, processing stops.", NULL, MB_OK );
					return;
				}
			}
		}
//
// JFE 9/23/01: End of changes
//
		else														// Found general data
		{															// to write out.
			if (!WriteToAllOutputFiles (m_InputBuffer))
				return;
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE:	Opens the files we are to read, finds the names of the output file
//				we are to create, and opens them also.
//
//   COMMENTS:
//
//	This function opens the file passed in the call to ConvertFile and the
//	control file named Export Objects.txt.  It will then read through Export Objects.txt
//	looking for required output files, and will create them as needed.
//
//    RETURN VALUES:
//		TRUE:	All files opened successfully
//		FALSE:	Otherwise
//
BOOL CBaseSplitter::OpenTheFiles()
{
//
//			Open the input file
//
	m_pIFStream = new ifstream;
	m_pIFStream->open (m_szInputFileName, (ios::in | ios::nocreate));
	if (m_pIFStream->fail())
	{
	    MessageBox( NULL, "Main input file open failed.", NULL, MB_OK );
	    return FALSE;
	}
//
//			Read the start of the file to see that it is an export file
//
	if (! ReadNextLine (m_pIFStream))
	{
	    MessageBox( NULL, "Could not read the indicated main input file\nProcessing stopped", NULL, MB_OK );
		m_pIFStream->close ();
	    return FALSE;
	}
	if (strcmp (m_InputBuffer, "#Generic Blender Export Data") != EQUAL)
	{
	    MessageBox( NULL, "Did not find expected export file starting with\n.#Generic Blender Export Data\nProcessing stopped", NULL, MB_OK );
		m_pIFStream->close ();
	    return FALSE;
	}
	strcpy (m_szHoldFileStart, m_InputBuffer);
//
//			Open the input control file
//
	m_pCtrlStream = new ifstream;
	m_pCtrlStream->open ("BlenderExportSplitCtrl.txt", (ios::in | ios::nocreate));
	if (m_pCtrlStream->fail())
	{
	    MessageBox( NULL, "Control input file open failed.", NULL, MB_OK );
	    return FALSE;
	}
//
//			Read through the control file looking for file names.  When found,
//			open an output file with the associated name.
//
	bool	bContinue	= true;
	int		nResult		= 0;
	char	szFilename	[MAX_PATH];

	while (bContinue)
	{
		if (! ReadNextLine (m_pCtrlStream))
		{
			CloseDown();
			return FALSE;	// File Name:
		}
		nResult = sscanf (m_InputBuffer, "%s %s", m_szWork, szFilename);
		if (stricmp (m_szWork, "FILE:") == EQUAL)					// found file name
		{
			if (!CreateOutputFile (szFilename))
			{
				CloseDown();
				return FALSE;	// File Name:
			}
		}
		else if (strnicmp (m_InputBuffer, "File Ends.", 10) == EQUAL)// Found EOF
		{
			m_pCtrlStream->close ();
			bContinue = false;
			continue;
		}
		else
		{
			nResult = sscanf (m_InputBuffer, "%s", m_szWork);
			if (nResult == 1)
			{
				if (!AddObjectToFile (m_szWork))
				{
					CloseDown();
					return FALSE;	// File Name:
				}
			}
			else
			{
				CloseDown();
				strcpy (m_szMessage, "Unknown input file line:\n");	// Don't know what we found
				strcat (m_szMessage, m_InputBuffer);
				MessageBox (NULL, m_szMessage, NULL, MB_OK );
				return FALSE;
			}
		}
	}
//
//			Write the first line of the file to each output file
//
	if (!WriteToAllOutputFiles (m_szHoldFileStart))
	{
		CloseDown();
		return FALSE;
	}

	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Reads the next line of useful input from the indicated input file
//
//   COMMENTS:
//
//	This method will read the next full line of text from the indicated input file.
//	It will place the results into m_InputBuffer
//	
//	  pInStream			Pointer to the stream to read the next line from
//	
//    RETURN VALUES:
//		TRUE	If it found a line to read and return
//		False	Otherwise
//
BOOL CBaseSplitter::ReadNextLine(ifstream * pInStream)
{
	pInStream->getline(m_InputBuffer, sizeof(m_InputBuffer));
	if (pInStream->fail())
	{
		MessageBox( NULL, "Input file read failed.", NULL, MB_OK );
		return FALSE;
	}
	return TRUE;
}


////////////////////////////////////////////////////////////////////////////////
//   PURPOSE:	Creates a new output file and saves its handle
//
//   COMMENTS:
//
//	This function creates a new output file named by the passed parameter, and
//	adds the resulting file handle to the m_FileMap using the file name as the
//	map's key.
//
//	  szFileName		The name of the output file to create
//	
//    RETURN VALUES:
//		TRUE:	File opened and saved successfully
//		FALSE:	Otherwise
//
BOOL CBaseSplitter::CreateOutputFile(char *szFilename)
{
//
//			Create and open the file
//
	HANDLE * pOutputFile = new HANDLE;

	if ((*pOutputFile = CreateFile((LPCTSTR) szFilename,
		GENERIC_WRITE,
		FILE_SHARE_READ | FILE_SHARE_WRITE,
		NULL,
		CREATE_ALWAYS,
		0,
		(HANDLE)NULL)) == (HANDLE)-1)
	{
		strcpy (m_szMessage, "Failed to create outout file: ");
		strcat (m_szMessage, szFilename);
		strcat (m_szMessage, ".");
	    MessageBox( NULL, m_szMessage, NULL, MB_OK );
		CloseDown();
	    return FALSE;
	}
//
//			Add the file name and handle to the file map
//
    char    *pNewName = new char [strlen(szFilename) + 1];
	lstrcpy (pNewName, szFilename);

	std::pair<FileMap_t::iterator, bool> InsertResult;
	InsertResult = m_FileMap.insert (FileMap_t::value_type (pNewName, pOutputFile));

	if (! InsertResult.second)
	{
		strcpy (m_szMessage, "Failed to add ");
		strcat (m_szMessage, szFilename);
		strcat (m_szMessage, " to the file pointer table.");
	    MessageBox( NULL, m_szMessage, NULL, MB_OK );
		return FALSE;
	}
	strcpy (m_szOutputFileName, szFilename);

	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE:	Add a new object to the current file name in m_ObjectMap
//
//   COMMENTS:
//
//	This function adds the passed object name to the object map using the
//	object name as the key and the current output file name as the key 
//	value thus linking this object to the current output file.  If the link
//	would be a duplicate, it adds nothing.
//
//	  szObjName			The name of the object to add to the current output file
//	
//    RETURN VALUES:
//		TRUE:	Object and file name link saved successfully or already present
//		FALSE:	Otherwise
//
BOOL CBaseSplitter::AddObjectToFile(char *szObjName)
{
//
//			Look for a duplicate entry, and exit if found
//
	ObjectMap_t::iterator iObjectMap = m_ObjectMap.find (szObjName);
	while (iObjectMap != m_ObjectMap.end())
	{
		if ((strcmp ((*iObjectMap).first,  szObjName) == EQUAL) &&
			(strcmp ((*iObjectMap).second, m_szOutputFileName) == EQUAL))
		{
			strcpy (m_szMessage, "Object ");
			strcat (m_szMessage, szObjName);
			strcat (m_szMessage, " is duplicated in file ");
			strcat (m_szMessage, m_szOutputFileName);
			strcat (m_szMessage, ", duplicate object not added to the file.");
			MessageBox( NULL, m_szMessage, NULL, MB_OK );
			return true;
		}
		iObjectMap++;
	}
//
//			Create a new entry for the current file
//    
	char    *pNewObjectName = new char [strlen(szObjName) + 1];
    char    *pNewFileName   = new char [strlen(m_szOutputFileName) + 1];
	lstrcpy (pNewObjectName, szObjName);
	lstrcpy (pNewFileName,   m_szOutputFileName);

	ObjectMap_t::iterator InsertResult;
	InsertResult = m_ObjectMap.insert (ObjectMap_t::value_type (pNewObjectName,
																pNewFileName));

	if (InsertResult == m_ObjectMap.end())
	{
		strcpy (m_szMessage, "Failed to add ");
		strcat (m_szMessage, szObjName);
		strcat (m_szMessage, " to the ");
		strcat (m_szMessage, m_szOutputFileName);
		strcat (m_szMessage, " table.");
	    MessageBox( NULL, m_szMessage, NULL, MB_OK );
		return FALSE;
	}

	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE:	Close all open files and empty all the maps
//
//   COMMENTS:
//
//	This function closes the main input file and all existing output file, 
//	frees the memory of all the objects with references in the file and
//	object maps, and then clears the maps themselves.
//
//    RETURN VALUES:
//		NONE
//
void CBaseSplitter::CloseDown()
{
//
//			Close the input files
//
	if (m_pIFStream->is_open())
		m_pIFStream->close ();
	if (m_pCtrlStream->is_open())
		m_pCtrlStream->close();
//
//			Close the output files
//
	HANDLE	*phFile;
	BOOL	bCloseOK;

	FileMap_t::iterator iFileMap = m_FileMap.begin();
	while (iFileMap != m_FileMap.end())
	{
		try
		{
			phFile = (HANDLE *)((*iFileMap).second);
			bCloseOK = CloseHandle (*phFile);
		}		
		catch (CFileException)
		{
			bCloseOK = false;
		}
		if (! bCloseOK)
		{
			strcpy (m_szMessage, "Failed to close file ");
			strcat (m_szMessage, (*iFileMap).first);
			strcat (m_szMessage, ", processing continues.");
			MessageBox( NULL, m_szMessage, NULL, MB_OK );
		}
		iFileMap++;
	}
//
//			Free the items controlled by the maps and erase the map entries.
//
	iFileMap = m_FileMap.begin();
	while (iFileMap != m_FileMap.end())
	{
		delete (char *)  ((*iFileMap).first);
		delete (HANDLE *)((*iFileMap).second);
		m_FileMap.erase (iFileMap);
		iFileMap = m_FileMap.begin();
	}

	ObjectMap_t::iterator iObjectMap = m_ObjectMap.begin();
	while (iObjectMap != m_ObjectMap.end())
	{
		delete (char *) ((*iObjectMap).first);
		delete (char *) ((*iObjectMap).second);
		m_ObjectMap.erase (iObjectMap);
		iObjectMap = m_ObjectMap.begin();
	}
}


////////////////////////////////////////////////////////////////////////////////
//   PURPOSE:	Write the text defining a mesh to the file defining that mesh
//
//   COMMENTS:
//
//	This function figures out which file(s) contain the indicated mesh, and then
//	cycle through the files writing each line of the mesh definition.  There is
//	usually a one-to-one relationship between meshes and files, but one mesh may
//	be used in more than one object definition file.
//
//	  szMeshName		The name of the mes to write out
//	
//    RETURN VALUES:
//		TRUE:	The mesh is properly written to each appropriate file.
//		FALSE:	Otherwise
//
BOOL CBaseSplitter::WriteOutMeshData(char *szMeshName)
{
	ObjectMap_t::iterator	iObjectMap;
	FileMap_t::iterator		iFileMap;

	HANDLE *phFile;
	bool	bFirstLine = true;
//
//			Write text to all the files containing this mesh
//
	while (true)		// while still reading/writing this mesh
	{
//
//			Look through the object map to file files that contain this object
//
		iObjectMap = m_ObjectMap.find (szMeshName);
		while ((iObjectMap != m_ObjectMap.end()) &&
			   (strcmp ((*iObjectMap).first,  szMeshName) == EQUAL))
		{
//
//				For each file, look up the file handle
//
			iFileMap = m_FileMap.find((*iObjectMap).second);
			if (iFileMap == m_FileMap.end())
			{
				strcpy (m_szMessage, "Could not find map entry for file ");
				strcat (m_szMessage, (*iObjectMap).second);
				strcat (m_szMessage, ".");
				MessageBox( NULL, m_szMessage, NULL, MB_OK );
				return false;
			}
			phFile = (HANDLE *)((*iFileMap).second);
//
//				Write out the appropriate text.
//
			if (bFirstLine)
			{
				if (!PerformTheWrite (*phFile, m_szHoldMeshStart))
					return false;
				bFirstLine = false;
			}
			if (!PerformTheWrite (*phFile, m_InputBuffer))
				return false;
			iObjectMap++;
		}
//
//			If the mesh just ended, return
//
		if (strnicmp (m_InputBuffer, "mesh ends.", 10) == EQUAL)
		{
			return true;
		}
//
//			Otherwise, read the next line of the mesh and continue
//
		if (! ReadNextLine (m_pIFStream))
		{
			CloseDown();
			return false;
		}
	}
	return true;
}


////////////////////////////////////////////////////////////////////////////////
//   PURPOSE:	Write the passed text to all the output files
//
//   COMMENTS:
//
//	This function writes all the passed text to all the output files.  This is
//	necessary for the start of each file before the mesh definitions begin.
//
//	  szText			Text to write to all of the output file.
//	
//    RETURN VALUES:
//		TRUE:	The output is properly written to each file.
//		FALSE:	Otherwise
//
BOOL CBaseSplitter::WriteToAllOutputFiles(char *szText)
{
	HANDLE	*phFile;

	FileMap_t::iterator iFileMap = m_FileMap.begin();
	while (iFileMap != m_FileMap.end())
	{
		phFile = (HANDLE *)((*iFileMap).second);
		if (!PerformTheWrite (*phFile, szText))
			return false;
		iFileMap++;
	}
	return true;
}


////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Write information to an output file
//
//   COMMENTS:
//
//	This method will perform the actual write to an output file and will check
//	for any resulting errors.  If errors occur, it will report them using the
//	passed message, and return a FALSE value.  Otherwise it will return a TRUE
//	value
//
//	  hOutputFile		The handle of the output file to use
//	  pData				Pointer to the data to write to the file
//	
//    RETURN VALUES:
//		TRUE	If no errors occur
//		False	Otherwise
//
BOOL CBaseSplitter::PerformTheWrite(HANDLE	hOutputFile,
								    char *	pData)
{
	DWORD dwSize;
	DWORD dwBytesWritten;

	if (strchr(pData, '\n') == NULL)	// add cr/lf if missing
		strcat (pData, "\r\n");
	dwSize = strlen (pData);
	WriteFile (hOutputFile, pData, dwSize, &dwBytesWritten, NULL);
	if (dwBytesWritten != dwSize)
	{
	    MessageBox( NULL, "Write to the output file failed.", NULL, MB_OK );
		return FALSE;
	}
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////
