////////////////////////////////////////////////////////////////////////////////
//	BaseSplitter.h:	The base (and only) class for splitting Blender export files
//
//	This class reads an Blender export text file containing the information
//	dumped from the objects in a Blender file, and a separate input control
//	text file that indicates the names of the individual Blender export file
//	to be created, and which objects go into which export file.  Some standard
//	information goes at the head of each export file, but the rest is made up of
//	the objects linked to that file.
//
//	This addresses the fact that the current Blender export facility does not
//	recognize "selected" objects, and is easier than having to select just the
//	right objects each time you want to export something anyway.
//
//	NOTE: This is assumed to be an open-source project, so individual method
//	notes appear before each method in the CPP file rather than here in the
//	.h file.
//
//	Copyright , Jack Emmerichs, 2001.
//
////////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_BASESPLITTER_H_INCLUDED_)
#define AFX_BASESPLITTER_H_INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <fstream.h>
#include <ios>

#pragma warning( disable : 4786 )  // Disable warning messages

////////////////////////////////////////////////////////////////////////////////
//	Define the less_s class used to compare two strings when using null
//	terminated C-style strings.
//
class less_s
{
	public:
	  bool operator () (const char* szStr1, const char* szStr2) const
	  {
		return (stricmp (szStr1, szStr2) < 0);
	  }
};


/////////////////////////////////////////////////////////////////////////////
//	Create STL map and multimap templates.  The File map holds a list of the
//	output files to create.  The Object multimap holds a list of the objects
//	associated with each file, and may assign one object to more than one file.
//
#include <map>

	typedef std::map	  <char *, HANDLE *, less_s> FileMap_t;
	typedef std::multimap <char *, char *,   less_s> ObjectMap_t;


/////////////////////////////////////////////////////////////////////////////
//	Define the Base File Splitter class that does most of the work.
//
class CBaseSplitter : public CObject  
{
public:
			CBaseSplitter	();
	virtual ~CBaseSplitter	();
	void	SplitInputFile	(char *	szFileName);


protected:
	BOOL	OpenTheFiles			();
	BOOL	ReadNextLine			(ifstream *	pInStream);
	BOOL	CreateOutputFile		(char *		szFilename);
	BOOL	AddObjectToFile			(char *		szObjName);
	void	CloseDown				();
	BOOL	WriteOutMeshData		(char *		szMeshName);
	BOOL	WriteToAllOutputFiles	(char *		szText);
	BOOL	PerformTheWrite			(HANDLE		hOutputFile,
									 char *		pData);

	FileMap_t	m_FileMap;							// file name map
	ObjectMap_t	m_ObjectMap;						// object/file relationship map
	ifstream*	m_pIFStream;						// Blender input file
	ifstream*	m_pCtrlStream;						// control input file
	char		m_InputBuffer			[10000];	// general input buffer
	char		m_szWork				[1024];		// general work string
	char		m_szMessage				[1024];		// general output message buffer
	char		m_szHoldFileStart		[1024];		// held first line of the input file
	char		m_szHoldMeshStart		[1024];		// held first line of a new mesh obj.
	char		m_szMeshObjectName		[1024];		// held mesh object name
	char		m_szInputFileName		[MAX_PATH];	// current input file name
	char		m_szOutputFileName		[MAX_PATH];	// current output file name

};

#endif // !defined(AFX_BASESPLITTER_H_INCLUDED_)
////////////////////////////////////////////////////////////////////////////////
